---
config:
  look: neo
  theme: redux
  layout: dagre
---
flowchart TD
{% for id, entity in feed.entities.items() %}
    {{ id }}["{{ entity.name }}"]:::{{ entity.super_type.short_name }}Class
    {%  for rid, relation in entity.get_immutable_relations().items() %}
    {{ relation.obj_from.id }} -- "{{ relation.name }}" --> {{ relation.obj_to.id }}
    {% endfor %}
{% endfor %}

{% for id, relation in feed.relations.items() %}
    {{ relation.obj_from.id }} -- "{{ relation.name }}" --> {{ relation.obj_to.id }}
{% endfor %}

{% for _, type in theme.types.items() %}
    classDef {{ type.short_name }}Class fill:{{ type.bg_color }}
{% endfor %}
